import json
import shutil
import pkg_resources
import pathlib
import os
import click
from dataclasses import dataclass
from accorder.utils.motw_client import calibre_to_json
from accorder.utils.configs import edit_config
from accorder.utils.bibtex import add_books

@dataclass(init=False)
class BuildPortable:
    buildportable_library_uuid: str
    buildportable_library_secret: str
    buildportable_librarian: str
    buildportable_local_directory: str
    buildportable_jsonpath: str
    buildportable_bibtex: str

    def copy_resources(self, pkg_dir, root_dest):
        for f in pkg_resources.resource_listdir("accorder", pkg_dir):
            if pkg_resources.resource_isdir("accorder", f"{pkg_dir}/{f}"):
                p = pathlib.Path(
                    pkg_resources.resource_filename("accorder", f"{pkg_dir}/{f}")
                )
                rel_path = pathlib.PurePath(*p.parts[p.parts.index("assets") + 1 :])
                dest_path = pathlib.PurePath(root_dest, rel_path)
                os.makedirs(str(dest_path), exist_ok=True)
                self.copy_resources(f"{pkg_dir}/{f}", root_dest)
            else:
                p = pathlib.Path(
                    pkg_resources.resource_filename("accorder", f"{pkg_dir}/{f}")
                )
                rel_path = str(pathlib.PurePath(*p.parts[p.parts.index("assets") + 1 :]))
                shutil.copy2(str(p), f"{root_dest}{rel_path}")

    def run_buildportable(self):
        if self.buildportable_bibtex:
            add_books(self.buildportable_bibtex, self.buildportable_local_directory)

        click.echo(">>>> BUILD PORTABLE CATALOG AS WEBAPP TO:")
        click.echo(f">>>> {self.buildportable_local_directory}")
        click.echo(f"\rbuilding {self.buildportable_librarian}'s portable.", nl=False)

        books = calibre_to_json(
            self.buildportable_library_uuid,
            self.buildportable_library_secret,
            self.buildportable_librarian,
            f"{self.buildportable_local_directory}metadata.db",
        )

        shutil.rmtree(
            f"{self.buildportable_local_directory}static/", ignore_errors=True
        )

        self.copy_resources("assets", self.buildportable_local_directory)

        # write first 48 books
        count = 1
        block = 24
        with open(f"{self.buildportable_local_directory}static/data{count}.js", "wb") as f:
            books_json = json.dumps({"portable": True, "total": len(books), "books": books[:block]})
            output = f"CALIBRE_BOOKS{count}={books_json}"
            f.write(output.encode())

        # write six more (2x96, 4x2304)
        count += 1
        for i in range(1, 3):
            for ii in range(2**i):
                end_block = block + 24**i*4
                with open(f"{self.buildportable_local_directory}static/data{count}.js", "wb") as f:
                    books_json = json.dumps({"portable": True, "books": books[block:end_block]})
                    output = f"CALIBRE_BOOKS{count}={books_json}"
                    f.write(output.encode())
                block = end_block

                count += 1
        # write last one with the rest
        with open(f"{self.buildportable_local_directory}static/data{count}.js", "wb") as f:
            books_json = json.dumps({"portable": True, "books": books[block:]})
            output = f"CALIBRE_BOOKS{count}={books_json}"
            f.write(output.encode())

        click.echo(
            f"\r{self.buildportable_librarian}'s portable catalog of {len(books)} books built."
        )
        click.echo(
            f"Check the webapp by opening in your browser:\n{self.buildportable_local_directory}BROWSE_LIBRARY.html"
        )
        click.echo("")

        if self.buildportable_jsonpath:
            bookz = {}
            for book in books:
                bookz[book["_id"]] = book

            with open(self.buildportable_jsonpath, "w") as f:
                f.write(json.dumps(bookz))

            click.echo(f"JSON file with all metadata rendered into {self.buildportable_jsonpath}")


def run_build(app_name, profile, directory, librarian, jsonpath, with_bibtex):
    config = edit_config(app_name, profile, directory=directory, librarian=librarian, bibtex=with_bibtex)

    r = BuildPortable()
    r.buildportable_library_uuid = config[profile]["calibre"]["library_uuid"]
    r.buildportable_library_secret = config[profile]["calibre"]["library_secret"]
    r.buildportable_librarian = config[profile]["calibre"]["librarian"]
    r.buildportable_local_directory = config[profile]["calibre"]["local_directory"]
    r.buildportable_jsonpath = jsonpath
    r.buildportable_bibtex = with_bibtex

    r.run_buildportable()



import os
import click
import configparser
import collections
import pathlib
import uuid


def read_config(app_name):
    cfg = os.path.join(click.get_app_dir(app_name), "accorder.ini")
    if not pathlib.Path(cfg).exists():
        click.echo(
            f"{cfg} doesn't exist. First, check out: `accorder configuration --help`."
        )
        click.get_current_context().exit()
    parser = configparser.RawConfigParser()
    parser.read([cfg])
    nested_dict = lambda: collections.defaultdict(nested_dict)
    conf = nested_dict()
    for section in parser.sections():
        profile, context = section.split("_")
        for key, value in parser.items(section):
            conf[profile][context][key] = value
    return conf


def write_config(app_name, options):
    cfg = os.path.join(click.get_app_dir(app_name), "accorder.ini")
    if not pathlib.Path(cfg).exists():
        os.makedirs(click.get_app_dir(app_name), exist_ok=True)

    parser = configparser.ConfigParser()
    parser.read(cfg)
    for option in options:
        if not parser.has_section(option["section"]):
            parser.add_section(option["section"])
        parser.set(option["section"], option["option"], option["value"])
    with open(cfg, "w") as f:
        parser.write(f)


def check_profile(config, profile):
    if profile not in list(config.keys()):
        click.echo(
            f"'{profile}' not found in the list of configured profiles: {', '.join(config.keys())}"
        )
        only_calibre = click.confirm(
            "Do you want to create a profile just for building the standalone, portable Calibre webapp?",
            default=True,
        )
        if not only_calibre:
            click.get_current_context().exit()


def check_sections(config, profile):
    return [
        section
        for section in ["meta", "calibre", "motw", "rsync", "tunnel", "rclone"]
        if section in config[profile].keys()
    ]


def edit_config(app_name, profile, directory=None, librarian=None, bibtex=None):
    config = read_config(app_name)
    check_profile(config, profile)

    if "local_directory" not in config[profile]["calibre"]:
        if not directory and not bibtex:
            local_directory = click.prompt(
                f"'Please, provide local Calibre's collection directory for {profile}",
                type=click.Path(exists=True, resolve_path=True, file_okay=False),
            )
            click.echo(local_directory)
        elif not directory and bibtex:
            local_directory = click.prompt(
                f"'Please, provide local Calibre's collection directory for {profile}",
                type=click.Path(resolve_path=True, dir_okay=True),
            )
        else:
            local_directory = directory
    else:
        if not directory:
            local_directory = config[profile]["calibre"]["local_directory"]
        else:
            local_directory = directory

    if local_directory[:-1] != "/":
        local_directory += "/"

    while "/" * 2 in local_directory:
        local_directory = local_directory.replace("//", "/")

    if not pathlib.Path(f"{local_directory}metadata.db").is_file() and not bibtex:
        click.echo(
            f"{local_directory} provided does not point to the root of Calibre library. Please, check again and provide the correct path."
        )
        click.get_current_context().exit()

    new_sections = [
        {
            "section": f"{profile}_calibre",
            "option": "local_directory",
            "value": local_directory,
        }
    ]

    if "librarian" not in config[profile]["calibre"]:
        if not librarian:
            librarian = click.prompt(
                f"'Please, provide librarian's name for {profile} (max. 42 chars)"
            )

    if librarian:
        new_sections.append(
            {
                "section": f"{profile}_calibre",
                "option": "librarian",
                "value": librarian[:48],
            }
        )

    if "library_uuid" not in config[profile]["calibre"]:
        new_sections.append(
            {
                "section": f"{profile}_calibre",
                "option": "library_uuid",
                "value": str(uuid.uuid4()),
            }
        )

    if "library_secret" not in config[profile]["calibre"]:
        new_sections.append(
            {
                "section": f"{profile}_calibre",
                "option": "library_secret",
                "value": uuid.uuid4().hex,
            }
        )

    write_config(app_name, new_sections)

    return read_config(app_name)



import os
import click
import urllib
import configparser
from accorder.utils.configs import read_config
from accorder.utils.configs import write_config
from accorder.utils.configs import edit_config
from accorder.utils.configs import check_sections


def run_configuration(app_name, configuration_id, show, replace):
    if configuration_id:
        if show:
            click.echo(click.get_current_context().get_help())
            click.get_current_context().exit()

        req_url = f"https://accorder.memoryoftheworld.org/{configuration_id}"
        req = urllib.request.Request(req_url)
        try:
            res = urllib.request.urlopen(req)
        except Exception as e:
            click.echo("")
            click.echo(f'Download from {req_url} didn\'t work. Please, check if {configuration_id} is correct CONFIGURATION_ID or check your internet connection and try again. Also, this might be the error: "{e}"')
            click.echo("")
            click.get_current_context().exit()

        res_code = res.getcode()
        if res_code == 200:
            config_string = res.read().decode()
        else:
            click.echo("")
            click.echo(f"Download from {req_url} didn't work. Please, check if {configuration_id} is correct or check your internet connection and try again.")
            click.echo("")
            click.get_current_context().exit()

        try:
            parser = configparser.ConfigParser()
            parser.read_string(config_string)
        except Exception as e:
            click.echo("")
            click.echo(f"{req_url} has corrupted configuration file. Check out with the administrator what could be done about that. Good luck! Also, this might be the error: {e}")
            click.echo("")
            click.get_current_context().exit()

        cfg = os.path.join(click.get_app_dir(app_name), "accorder.ini")

        new_sections = []
        for section in parser.sections():
            for option, value in parser.items(section):
                new_sections.append({
                    "section": section,
                    "option": option,
                    "value": value
                })

        if replace:
            with open(cfg, "w") as f:
                f.write(config_string)
        else:
            write_config(app_name, new_sections)
        click.get_current_context().exit()

    if show:
        config = read_config(app_name)
        config_path = os.path.join(click.get_app_dir(app_name), "accorder.ini")
        click.echo(f"CONFIGURATION FILE: {config_path}")
        click.echo("")

        if not config.keys():
            click.echo("")
            click.echo(
                f"{config_path} has no profile configured. First, you probably want to initialize your setup with:\n`accorder configuration CONFIGURATION_ID`\nor if you just want a profile to build a standalone, portable Calibre library run:\n`accorder build PROFILE`"
            )
            click.echo("")

        for profile in config.keys():
            sections = check_sections(config, profile)
            if not sections:
                edit_config(app_name, profile)

            if "calibre" in sections:
                for option in [
                    "librarian",
                    "local_directory",
                    "library_uuid",
                    "library_secret",
                ]:
                    if option not in config[profile]["calibre"]:
                        edit_config(app_name, profile)
            else:
                edit_config(app_name, profile)
            config = read_config(app_name)
            librarian = config[profile]["calibre"]["librarian"]
            local_directory = config[profile]["calibre"]["local_directory"]
            click.echo(f"..PROFILE: {profile}")
            click.echo(f"....LIBRARIAN: {librarian}")
            click.echo(f"....CALIBRE LIBRARY: {local_directory}")

            upload = []
            if "rsync" in sections:
                upload.append("RSYNC")
            if "rclone" in sections:
                upload.append("RCLONE")
            if upload:
                upload = ", ".join(upload)
                click.echo(f"....UPLOAD: {upload}")

            if "motw" in sections:
                motw_url = f'https://{config[profile]["motw"]["subdomain"]}.{config[profile]["motw"]["domain"]}'
                click.echo(f"....SHARED AT: {motw_url}")

            click.echo("")

    if replace and not configuration_id:
        click.echo(click.get_current_context().get_help())
